%% FAST interface: configure, launch, read outputs & analyse
% Fermin Reinicke
% UPV/EHU - Energia Berriztagarrien Ingeniaritza

clear
close all
clc

%% ---- Prepare the main input file: OC3_Hywind.fst ----------------------

% Define the file's name
MainConfig = 'OC3_Hywind';
InputFileName = [MainConfig, '.fst'];

% Set the main parameters
TMax = '650';           % Total run time [s]
DT = '0.0125';          % Recommended module time step [s]
CompElas = '1';         % Compute structural dynamics (switch)
                            % 1: ElastoDyn
                            % 2: ElastoDyn + BeamDyn for blades
CompInfl = '1';         % Compute inflow wind velocities (switch)
                            % 0: still air
                            % 1: InflowWind
                            % 2: external from OpenFOAM
CompAero = '2';         % Compute aerodynamic loads (switch)
                            % 0: None
                            % 1: AeroDyn v14
                            % 2: AeroDyn v15
CompServ = '1';         % Compute control and electrical-drive dynamics (switch)
                            % 0: None
                            % 1: ServoDyn
CompHydr = '1';         % Compute hydrodynamic loads (switch)
                            % 0: None
                            % 1: HydroDyn
CompSub = '0';          % Compute sub-structural dynamics (switch)
                            % 0: None
                            % 1: SubDyn
CompMoor = '1';         % Compute Mooring system (switch)
                            % 0: None
                            % 1: MAP++
                            % 2: FEAMooring
                            % 3: MoorDyn
                            % 4: OrcaFlex
% Input files' names
EDF = '"5MW_Baseline/NRELOffshrBsline5MW_OC3Hywind_ElastoDyn.dat"';
BDBldF = '"5MW_Baseline/NRELOffshrBsline5MW_BeamDyn.dat"';
InflowF = '"5MW_Baseline/NRELOffshrBsline5MW_InflowWind.dat"';
AeroF = '"5MW_Baseline/NRELOffshrBsline5MW_OC3Hywind_AeroDyn15.dat"';
ServoF = '"5MW_Baseline/NRELOffshrBsline5MW_OC3Hywind_ServoDyn.dat"';
HydroF = '"5MW_Baseline/NRELOffshrBsline5MW_OC3Hywind_HydroDyn.dat"';
SubF = '"unused"';
MooringF = '"5MW_Baseline/NRELOffshrBsline5MW_OC3Hywind_MAP.dat"';
IceF = '"unused"';
InputF = {EDF;BDBldF;InflowF;AeroF;ServoF;HydroF;SubF;MooringF;IceF};   % All filenames in a vector
% Linearization parameters
Linearize = 'False';    % Linearization analysis
%                           switch: T/F
NLinTimes = '2';        % Number of times to linearize
LinTimes = '30, 60';    % List of times at which to linearize [s]
LinInps = '1';          % Inputs included in linearization (switch)
                            % 0: none
                            % 1: standard
                            % 2: all module inputs (debug)
LinOuts = '1';          % Outputs included in linearization (switch)
                            % 0: none
                            % 1: from OutList(s)
                            % 2: all module outputs (debug)

% Change the parameters in the OC3_Hywind.fst input file: 'chFst'
chFst(TMax, DT, CompElas, CompInfl, CompAero, CompServ, CompHydr, CompSub, CompMoor, InputF,...
      Linearize, NLinTimes, LinTimes, LinInps, LinOuts);

%% ---- Prepare HydroDyn --------------------------------------------------

% Set the main parameters
rho_W = '1025';       % [kg/m^3]
Depth = '320';        % [m]
WaveM = '2';          % 0: No waves
                      % 1: Regular waves
                      % 2: Irregular waves
Hs = '2';             % [m]
Tp = '12';            % [s]
GAMMA = '"DEFAULT"';    % [-] used only when WaveM=2
THETA = '0';          % [] Incident wave propagation heading direction
CurrM = '0';          % 0: No current
                      % 1: Standard current
                      % 2: User-defined from routine UserCurrent
CurrSSV = '0';        % Sub-surface current velocity at still water level  [m/s]. Used only when CurrMod=1

% Change the parameters in the HydroDyn.dat input file: 'chHydroDyn'
chHydroDyn(rho_W, Depth, WaveM, Hs, Tp, GAMMA, THETA, CurrM, CurrSSV);

% Create some chars to be able to save results in an orderly manner
if WaveM == '0'
    WaveM_str = 'No';
    H_str = '';
    T_str = '';
elseif WaveM == '1'
    WaveM_str = 'Reg';
    H_str = ['Hw=' Hs 'm'];
    T_str = ['Tw=' Tp 's'];
elseif WaveM == '2'
    WaveM_str = 'Irr';
    H_str = ['Hs=' Hs 'm'];
    T_str = ['Tp=' Tp 's'];
end

%% ---- Prepare AeroDyn ---------------------------------------------------

% Set the main parameters
rho_A = '1.225';      % [kg/m^3] air density
nu_A = '1.464e-05';   % [m^2/s] kinematic viscosity

% Change the parameters in the AeroDyn15.dat input file: 'chAeroDyn'
chAeroDyn(rho_A, nu_A);

%% ---- Prepare ElastoDyn -------------------------------------------------

% Set the main parameters
InBlPitch = '0';        % [] initial pitch for both the 3 blades
InRotSpeed = '7.00';    % [rpm] initial or fixed rotor speed
InNacYaw = '0';         % [] initial or fixed nacelle-yaw angle

% Change the parameters in the AeroDyn15.dat input file: 'chAeroDyn'
chElastoDyn(InBlPitch, InRotSpeed, InNacYaw);

%% ---- Prepare InflowWind ------------------------------------------------

% Set the main parameters
WindType = '3';           % switch for wind file type
                              % 1: steady
                              % 2: uniform
                              % 3: binary TurbSim FF
                              % 4: binary Bladed-style FF
                              % 5: HAWC format
                              % 6: User defined
HWindSpeed_1 = '12';      % Horizontal windspeed [m/s]
RefHt_12 = '90';          % Reference height for horizontal wind speed [m]
PLexp_1 = '0.2';          % Power law exponent
Filename_23 = 'Wind\90m_9mps_650s__IECKAI_IEC61400-3B_LOG__01.bts';   % Filename of:
                                                                          % time series data for uniform wind field (when WindType = 2)
                                                                          % the full field wind file to use (.bts) (when WindType = 3)
RefLength_2 = '125.88';   % Reference length for linear horizontal and vertical sheer [-]
FileRoot_4 = 'Wind\90m_10mps_800s__01\90m_10mps_800s__01';            % Rootname of the full-field wind file to use (.wnd, .sum)

% Change the parameters in the InflowWind.dat input file: 'chInflowWind'
chInflowWind(WindType, HWindSpeed_1, RefHt_12, PLexp_1, Filename_23, RefLength_2, FileRoot_4);

% Create some chars to be able to save results in an orderly manner
if WindType == '1'
    WindT_str = 'Steady';
    U_str = ['-U=' HWindSpeed_1 'mps'];
elseif WindType == '2'
    WindT_str = 'Uniform';
    U_str = '';
elseif WindType == '3'
    WindT_str = 'TurbSim';
    U_str = '';
elseif WindType == '4'
    WindT_str = 'binBladedStyle';
    U_str = '';
elseif WindType == '5'
    WindT_str = 'HAWC';
    U_str = '';
end

% -------------------------------------------------------------------------
%% ---- Launch FAST from Matlab -------------------------------------------
system(['FASTinput\FAST_x64.exe ','FASTinput\',InputFileName])
 
% -------------------------------------------------------------------------
%% ---- Move the output files to the output directory ---------------------

% Define the name of the simulation
simNumb = '11.04';
SimName = sprintf('%s_%sWaves-%s-%s_%sWind%s',simNumb,WaveM_str,H_str,T_str,WindT_str,U_str);

% Create a directory for the current simulation
if exist(['FASToutput\',SimName],'dir')==0   % exist('objektua')=0 izango da objektu hori (fitxategia, karpeta) ez badago
    mkdir(['FASToutput\',SimName]);          % 'mkdir(PARNTDIR,NEWDIR)' funtzioak 'NEWDIR' karpeta sortuko du 'PARENTDIR' helbidean
end

% Define the new directory as the output path
OutputPath = ['FASToutput\' SimName];

% Define the name for the output data file (full name with relative path)
OutputFileName=[OutputPath,filesep,SimName,'.outb'];
    if exist(OutputFileName,'file')==2      % exist()=2 izateko, '.m','.mlx','mlapp' edo Matlab-en erregistratu gabeko luzapena
                                            % ('.txt','.fig','.mat') eduki behar du 'OutputFileName' helbideari dagokion fitxategiak
                                            % '.out' luzapena ere multzo honetan sartzen da
        delete(OutputFileName);
    end

% Output fitxategiak 'output' karpetara mugitu: 'movefile(SOURCE,DESTINATION)' funtzioa
%movefile(['FASTinput\' MainConfig '.out'],[OutputPath,filesep,SimName,'.out'])     % '.out'-a
movefile(['FASTinput\' MainConfig '.outb'],OutputFileName)                          % '.outb'-a
fileList=dir('FASTinput\*.sum');                                                    % '.sum' guztiak
        % 'dir' funtzioak karpeta baten barruko edukia zerrendatzen du.
        % 'D=dir('N')' exekutatzean, karpeta barruko fitxategi bakoitzaren informazioa 6 objektutan sailkatuko da.
        % Objektuok ('name','folder','date','bytes','isdir','datenum'), 'structure' motako aldagai bakarrean gordeko dira, D-n hain zuzen ere.
for fli=1:length(fileList)
    if exist([OutputPath fileList(fli,1).name],'file')==2
        delete([OutputPath fileList(fli,1).name]);
    end
    movefile(['FASTinput\' fileList(fli,1).name], OutputPath)
end

%% ---- Read output of the FAST simulation --------------------------------

% Output datuak inportatu: 'ReadFASTbinary' komandoa
[Channels, Names, Units, FileID] = ReadFASTbinary(OutputFileName);

% Searching and organising the interesting outputs
Time = Channels(:,1);
Wind_UVW = Channels(:,2:4);         % Wind X, Y and Z components [m/s]
BldPitch = Channels(:,5:7);
RotSpeed = Channels(:,9);
RotTorq = Channels(:,70);
RotThrust = Channels(:,67);         % Low Speed Shaft thrust [kN]
RotPower = Channels(:,90)/1000;     % Power absorbed by the rotor [kW]
GenSpeed = Channels(:,10);
GenTorq = Channels(:,94);
GenPower = Channels(:,93);
TSR = Channels(:,89);               % Tip Speed Ratio [-]
Cp = Channels(:,91);
Ct = Channels(:,92);
NacYawP = Channels(:,11);           % Nacelle yaw angle []
NacYawV = Channels(:,12);           % Nacelle yaw angular velocity [/s]
PtfmDisp = Channels(:,28:30);       % Platform displacements [m]: surge(x), sway(y) and heave(z)
PtfmRot = Channels(:,31:33);        % Platform rotations []: roll(x), pitch(y) and yaw(z)
eta = Channels(:,95);               % Wave elevation at the platform reference point

%% ---- Save Results ------------------------------------------------------

save([OutputPath, filesep, SimName, '.mat'])

% -------------------------------------------------------------------------
%% ---- Plot Results ------------------------------------------------------

% Hasierakoak: U, BldPitch, GenSpeed, GenPower
haizea=figure('Name','U','Units','normalized','outerposition',[0 0 1 1]);   % irudi-leihoa sortu, bere izenarekin eta tamaina handian
plot(Time, Wind_UVW(:,1))
grid on
axis([50 str2double(TMax) 0 30])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('U [m/s]', 'FontWeight', 'bold')
title('AXIAL WIND SPEED vs. TIME', 'FontSize', 13)
print(haizea, [OutputPath,filesep,simNumb,'.plot_','U','.png'], '-dpng', '-r200')
% bestela, saveas(haizea,[non,simNumb,'.plot_','U.png'])

pitch=figure('Name', 'Blades pitch angle', 'Units', 'normalized', 'outerposition', [0 0 1 1]);   % irudi-leihoa sortu...
plot(Time, BldPitch(:,1))
grid on
axis([50 str2double(TMax) 0 30])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Blade pitch angle []', 'FontWeight', 'bold')
title('BLADE PITCH vs. TIME', 'FontSize', 13)
print(pitch, [OutputPath,filesep,simNumb,'.plot_','BldPitch','.png'], '-dpng', '-r200')
% bestela, saveas(pitch,[non,simNumb,'.plot_','BldPitch.png'])

Ngen=figure('Name', 'Generator speed', 'Units', 'normalized', 'outerposition', [0 0 1 1]);   % irudi-leihoa sortu...
plot(Time, GenSpeed)
grid on
axis([50 inf 500 1500])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('High speed shaft speed [rpm]', 'FontWeight', 'bold')
title('GENERATOR SPEED vs. TIME', 'FontSize', 13)
print(Ngen, [OutputPath,filesep,simNumb,'.plot_','GenSpeed','.png'], '-dpng', '-r200')
% bestela, saveas(Ngen,[non,simNumb,'.plot_','GenSpeed.png'])

Pout=figure('Name', 'Generator power', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time, GenPower)
grid on
axis([50 inf 0 7e3])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Generator power [kW]', 'FontWeight', 'bold')
title('OUTPUT POWER vs. TIME', 'FontSize', 13)
print(Pout, [OutputPath,filesep,simNumb,'.plot_','GenPower','.png'], '-dpng', '-r200')
% bestela, saveas(Pout,[non,simNumb,'.plot_','GenPower.png'])

% TSR eta Cp
LAMBDA=figure('Name', 'TSR', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time, TSR)
grid on
axis([50 inf 3 17])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('TipSpeedRatio [-]', 'FontWeight', 'bold')
title('TSR vs. TIME', 'FontSize', 13)
print(LAMBDA, [OutputPath,filesep,simNumb,'.plot_','TSR','.png'], '-dpng', '-r200')

CP=figure('Name', 'C_P', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time, Cp)
grid on
axis([50 inf 0 0.7])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Cp [-]', 'FontWeight', 'bold')
title('C_P vs. TIME', 'FontSize', 13)
print(CP, [OutputPath,filesep,simNumb,'.plot_','Cp','.png'], '-dpng', '-r200')

% Generator Torque
Tout=figure('Name', 'Generator torque', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time, GenTorq)
grid on
axis([50 inf 0 45])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Electromagnetic torque [kNm]', 'FontWeight', 'bold')
title('GENERATOR TORQUE vs. TIME', 'FontSize', 13)
print(Tout, [OutputPath,filesep,simNumb,'.plot_','GenTorque','.png'], '-dpng', '-r200')

% Inflow Wind & Wave
%figure
%plot(Time,Wind_UVW)
%grid on
%xlabel('t [s]')
%ylabel('Wind speed [m/s]')
%legend('U (X_i direction)','V (Y_i direction)','W (Z_i direction)')
%figure
%plot(Time,eta)
%grid on
%xlabel('Time [s]')
%ylabel('Wave elevation [m]')

% Wind Turbine Motion & Deformation (Platform)
EH = figure('Name', 'ETA vs. Heave', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time,eta,Time,PtfmDisp(:,3))
grid on
xlim([50 inf])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('[m]', 'FontWeight', 'bold')
title('\eta & PLATFORM HEAVE', 'FontSize', 13)
legend ('\eta','Heave')
print(EH, [OutputPath,filesep,simNumb,'.plot_','ETA_vs_PtfmHeave','.png'], '-dpng', '-r200')

PDis = figure('Name', 'Platform displacements', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time,PtfmDisp(:,1:3))
grid on
xlim([50 inf])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Translation [m]', 'FontWeight', 'bold')
title('PLATFORM DISPLACEMENTS', 'FontSize', 13)
legend ('Surge','Sway','Heave')
print(PDis, [OutputPath,filesep,simNumb,'.plot_','PtfmDisplacements','.png'], '-dpng', '-r200')

PRot = figure('Name', 'Platform rotations', 'Units', 'normalized', 'outerposition', [0 0 0.98 1]);   % irudi-leihoa sortu...
plot(Time,PtfmRot(:,1:3))
grid on
xlim([50 inf])
xlabel('Time [s]', 'FontWeight', 'bold')
ylabel('Rotation []', 'FontWeight', 'bold')
title('PLATFORM ROTATIONS', 'FontSize', 13)
legend ('Roll','Pitch','Yaw')
print(PRot, [OutputPath,filesep,simNumb,'.plot_','PtfmRotations','.png'], '-dpng', '-r200')


%% ---- Wind FFT analysis -------------------------------------------------

fft_EHU(Wind_UVW(:,1), Time, 50, 650)
axis([0 0.4 0 1])
text(0.13, 0.85, 'WIND FILE: 90m_15mps_650s__IECKAI_IEC61400-1A_PL__01.bts')